from .base import BaseBuilder

sitemap_str = """<?xml version="1.0" encoding="UTF-8">
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9" 
  xmlns:image="http://www.google.com/schemas/sitemap-image/1.1" 
  xmlns:video="http://www.google.com/schemas/sitemap-video/1.1">
{urls}
</urlset>
"""

url_str = """  <url>
    <loc>{url}</loc>
  </url>
"""

class SitemapBuilder(BaseBuilder):

    def build_sitemap(self, language, links):
        urls = []
        for link in links:
            urls.append(url_str.format(url=self.site.full_href(language, link)))
        sitemap = sitemap_str.format(urls="\n".join(urls))
        dst = self.site.get_dst('sitemap', language, extension='xml')
        self.site.write(sitemap, dst)

    def build(self):
        for language, links in self.site.links.items():
            self.build_sitemap(language, links)



class BaseProcessor(object):

    def translate(self, key, *args, **kwargs):
        return self.site.translate(self.language, key, *args, **kwargs)

    def file(self, filename):
        return self.site.request('static_file', filename)

    def href(self, href):
        return self.site.href(self.language, href)

    def __init__(self, site, params, language):
        self.site = site
        self.params = params
        self.language = language

    


import yaml
import os

def update(d, ud, overwrite=True):
    for key, value in ud.items():
        if key not in d:
            d[key] = value
        elif isinstance(value, dict):
            update(d[key], value, overwrite=overwrite)
        elif isinstance(value, list) and isinstance(d[key], list):
            d[key] += value
        else:
            if key in d and not overwrite:
                return
            d[key] = value

def load_includes(config, include_path):
    if isinstance(config, dict):
        d = {}

        def load_include(key='$include'):
            value = config[key]
            path = os.path.abspath(os.path.join(os.path.dirname(include_path[-1]), value))
            if path in include_path:
                raise ValueError("Recursive import of {} (path: {})"\
                    .format(path, '->'\
                    .join(['"{}"'\
                    .format(s) for s in include_path])))
            nd = load_config(path, include_path=include_path+[path])
            if nd:
                update(d, nd)

        if '$include' in config:
            load_include('$include')
        for key, value in config.items():
            if key == '$include':
                continue
            result = load_includes(value, include_path=include_path)
            if key in d and isinstance(d[key], dict):
                update(d[key], result)
            else:
                d[key] = result
        if '$include-after' in config:
            load_include('$include-after')
        return d
    elif isinstance(config, list):
        return [load_includes(i, include_path=include_path) for i in config]
    return config
            

def load_config(filename, include_path=None):
    if include_path is None:
        include_path = [os.path.abspath(filename)]
    with open(filename) as input_file:
        config = yaml.load(input_file.read())
    return load_includes(config, include_path=include_path)
